#!/usr/bin/env python

# Copyright 2013 Canonical Ltd. All rights reserved.
# Author: Brad Marshall <brad.marshall@canonical.com>

import base64
import os
import sys
import json
import yaml
import subprocess

local_copy = os.path.join(
    os.path.dirname(os.path.abspath(os.path.dirname(__file__))),
    "lib", "charm-helpers")
if os.path.exists(local_copy) and os.path.isdir(local_copy):
    sys.path.insert(0, local_copy)


from charmhelpers.core.host import (
    apt_install,
    mkdir,
    service,
    service_start,
    service_stop,
    write_file,
)

from charmhelpers.core.hookenv import (
    Hooks,
    config,
    open_port,
    relation_get,
    relation_id,
    remote_unit,
    unit_get,
)

hook_dir = os.path.abspath(os.path.dirname(__file__))
charm_dir = os.path.dirname(hook_dir)

listen_port = config().get('listen_port')
listenip = config().get('listenip')
pid_file = config().get('pid_file')
log_directory = config().get('log_directory')
log_format = config().get('log_format')
network_config = config().get('network')
user_config = config().get('user')
backlog_lines = config().get('backlog_lines')
backlog_msg_only = config().get('backlog_msg_only')
backlog_always = config().get('backlog_always')
ssl_cert = config().get('ssl_cert')
bip_conf = '/etc/bip.conf'
bip_defaults = '/etc/default/bip'

required_pkgs = [
    'bip',
    'python-jinja2',
]

hooks = Hooks()


def update_config():
    from jinja2 import Environment, FileSystemLoader
    if network_config:
        ## Try originally supported JSON formatted config
        try:
            network = json.loads(network_config)
        ## else use YAML (current):
        except ValueError:
            network = yaml.load(network_config)
    else:
        network = {}
    if user_config:
        ## Try originally supported JSON formatted config
        try:
            user = json.loads(user_config)
        ## else use YAML (current):
        except ValueError:
            user = yaml.load(user_config)
    else:
        user = {}

    template_env = Environment(
        loader=FileSystemLoader(os.path.join(os.environ['CHARM_DIR'],
                                'templates')))
    templ_vars = {
        'listen_port': listen_port,
        'listenip': listenip,
        'pid_file': pid_file,
        'log_directory': log_directory,
        'log_format': log_format,
        'network': network,
        'user': user,
        'backlog_lines': backlog_lines,
        'backlog_msg_only': backlog_msg_only,
        'backlog_always': backlog_always,
        'ssl_cert': ssl_cert,
    }
    template = template_env.get_template('bip_conf.template').render(templ_vars)
    with open(bip_conf, 'w') as bip_conf_config:
        bip_conf_config.write(str(template))


def setup_ssl():
    ssl_cert_path = "/var/lib/bip/bip.pem"
    if ssl_cert == "SELFSIGNED":
        env = os.environ.copy()
        env['OPENSSL_CN'] = "BIP SERVER SELFSIGNED"
        env['OPENSSL_PUBLIC'] = unit_get("public-address")
        env['OPENSSL_PRIVATE'] = unit_get("private-address")

        subprocess.check_call(["openssl", "req", "-new", "-x509", "-days",
                               "600", "-nodes",
                               '-config', os.path.join(os.environ['CHARM_DIR'], 'openssl.cnf'),
                               "-out", ssl_cert_path,
                               "-keyout", ssl_cert_path], env=env)
    else:
        with open(ssl_cert_path, 'w') as fp:
            fp.write(base64.decode(ssl_cert))
    # fix permissions
    subprocess.check_call(["chown", "root:bip", ssl_cert_path])
    subprocess.check_call(["chmod", "660", ssl_cert_path])


@hooks.hook()
def install():
    apt_install(required_pkgs, options=['--force-yes'])
    enabled_str = "ENABLED=1"
    write_file(bip_defaults, enabled_str)
    update_config()


@hooks.hook('config-changed')
def config_changed():
    if ssl_cert:
        setup_ssl()
    update_config()
    service('reload', 'bip')
    open_port(listen_port)


@hooks.hook('start')
def start():
    service_start('bip')


@hooks.hook('stop')
def stop():
    service_stop('bip')


@hooks.hook("upgrade-charm")
def upgrade_charm():
    install()


if __name__ == "__main__":
    hooks.execute(sys.argv)
